@@ -12,11 +12,13 @@ from ruamel.yaml import YAML
 from pcvs import (NAME_BUILD_CONF_FN, NAME_BUILD_RESDIR, NAME_BUILDFILE,
                   NAME_BUILDIR, NAME_SRCDIR)
 from pcvs.backend import bank as pvBank
+from pcvs.backend import spack as pvSpack
 from pcvs.helpers import communications, criterion, log, utils
 from pcvs.helpers.exceptions import RunException
 from pcvs.helpers.system import MetaConfig, MetaDict
 from pcvs.orchestration import Orchestrator
 from pcvs.plugins import Plugin
+from pcvs import testing
 from pcvs.testing.tedesc import TEDescriptor
 from pcvs.testing.testfile import TestFile, load_yaml_file
 
@@ -68,7 +70,7 @@ def display_summary(the_session):
     log.manager.print_item("Verbosity: {}".format(
         log.manager.get_verbosity_str().capitalize()))
     log.manager.print_section("User directories:")
-    width = max([len(i) for i in cfg.dirs])
+    width = max([0] + [len(i) for i in cfg.dirs])
     for k, v in cfg.dirs.items():
         log.manager.print_item("{:<{width}}: {:<{width}}".format(
             k.upper(),
@@ -124,7 +126,10 @@ def process_main_workflow(the_session=None):
     else:
         log.manager.print_section("Load Test Suites")
         start = time.time()
-        process()
+        if MetaConfig.root.validation.dirs:
+            process_files()
+        if MetaConfig.root.validation.spack_recipe:
+            process_spack()
         end = time.time()
         log.manager.print_section(
             "===> Processing done in {:<.3f} sec(s)".format(end-start))
@@ -143,8 +148,9 @@ def process_main_workflow(the_session=None):
         return 0
 
     log.manager.print_header("Execution")
-    rc += MetaConfig.root.get_internal('orchestrator').run(the_session)
-
+    run_rc = MetaConfig.root.get_internal('orchestrator').run(the_session)
+    rc += run_rc if isinstance(run_rc, int) else 1
+    
     log.manager.print_header("Finalization")
     # post-actions to build the archive, post-process the webview...
     terminate()
@@ -152,13 +158,12 @@ def process_main_workflow(the_session=None):
     bank_token = valcfg.target_bank
     if bank_token is not None:
         bank = pvBank.Bank(token=bank_token)
-        pref_proj = bank.preferred_proj
+        pref_proj = bank.default_project
         if bank.exists():
             log.manager.print_item("Upload results to bank: '{}{}'".format(
                 bank.name.upper(),
                 " (@{})".format(pref_proj) if pref_proj else ""
             ))
-            bank.connect_repository()
             bank.save_from_buildir(
                 None,
                 os.path.join(valcfg.output)
@@ -246,8 +251,6 @@ def prepare():
     # TODO: replace resource here by the one read from config
     TEDescriptor.init_system_wide('n_node')
 
-    log.manager.print_item("Init the global Orchestrator")
-    MetaConfig.root.set_internal('orchestrator', Orchestrator())
 
     if valcfg.enable_report:
         log.manager.print_section("Connection to the Reporting Server")
@@ -261,6 +264,9 @@ def prepare():
             log.manager.print_item("Listening on {}".format(comman.endpoint))
         MetaConfig.root.set_internal('comman', comman)
 
+    log.manager.print_item("Init the global Orchestrator")
+    MetaConfig.root.set_internal('orchestrator', Orchestrator())
+
     log.manager.print_item("Save Configurations into {}".format(valcfg.output))
     conf_file = os.path.join(valcfg.output, NAME_BUILD_CONF_FN)
     with open(conf_file, 'w') as conf_fh:
@@ -316,7 +322,7 @@ def find_files_to_process(path_dict):
     return (setup_files, yaml_files)
 
 
-def process():
+def process_files():
     """Process the test-suite generation.
 
     It includes walking through user directories to find definitions AND
@@ -346,6 +352,24 @@ def process():
                         )
         raise RunException.TestUnfoldError("See previous errors above.")
 
+def process_spack():
+    
+    if not shutil.which('spack'):
+        log.manager.warn("Unable to parse Spack recipes without having Spack in $PATH")
+        return
+    log.manager.print_item("Build test-bases from Spack recipes")
+    label = "spack"
+    path = "/spack"
+    MetaConfig.root.validation.dirs[label] = path
+    
+    _, _, rbuild, _ = testing.generate_local_variables(label, '')
+    utils.create_or_clean_path(rbuild, dir=True)
+
+    with log.progbar(MetaConfig.root.validation.spack_recipe) as itbar:
+        for spec in itbar:
+            _, _, _, cbuild = testing.generate_local_variables(label, spec)
+            utils.create_or_clean_path(cbuild, dir=True)
+            pvSpack.generate_from_variants(spec, label, spec)
 
 def build_env_from_configuration(current_node, parent_prefix="pcvs"):
     """create a flat dict of variables mapping to the actual configuration.
@@ -412,7 +436,7 @@ def process_dyn_setup_scripts(setup_files):
     with log.progbar(setup_files, print_func=print_progbar_walker) as itbar:
         for label, subprefix, fname in itbar:
             log.manager.debug("process {} ({})".format(subprefix, label))
-            base_src, cur_src, base_build, cur_build = utils.generate_local_variables(
+            base_src, cur_src, base_build, cur_build = testing.generate_local_variables(
                 label, subprefix)
 
             # prepre to exec pcvs.setup script
@@ -486,7 +510,7 @@ def process_static_yaml_files(yaml_files):
     log.manager.info("Iteration over files")
     with log.progbar(yaml_files, print_func=print_progbar_walker) as iterbar:
         for label, subprefix, fname in iterbar:
-            _, cur_src, _, cur_build = utils.generate_local_variables(
+            _, cur_src, _, cur_build = testing.generate_local_variables(
                 label, subprefix)
             if not os.path.isdir(cur_build):
                 os.makedirs(cur_build)
@@ -501,6 +525,7 @@ def process_static_yaml_files(yaml_files):
                 obj.process()
                 obj.flush_sh_file()
             except Exception as e:
+                raise e
                 err.append((f, e))
                 log.manager.info("{} (failed to parse): {}".format(f, e))
     return err
