@@ -7,7 +7,7 @@ from pcvs.helpers.criterion import Criterion, Serie
 from pcvs.helpers.exceptions import TestException
 from pcvs.helpers.system import MetaConfig, MetaDict
 from pcvs.testing.test import Test
-
+from pcvs import testing
 
 def detect_source_lang(array_of_files):
     """Determine compilation language for a target file (or list of files).
@@ -172,7 +172,7 @@ class TEDescriptor:
         self._te_label = label
         self._te_subtree = subprefix
 
-        _, self._srcdir, _, self._buildir = utils.generate_local_variables(
+        _, self._srcdir, _, self._buildir = testing.generate_local_variables(
             label,
             subprefix
         )
@@ -189,14 +189,19 @@ class TEDescriptor:
         self._run = MetaDict(node.get('run', None))
         self._validation = MetaDict(node.get('validate', None))
         self._artifacts = MetaDict(node.get('artifact', None))
+        self._metrics = MetaDict(node.get('metric', None))
         self._template = node.get('group', None)
         self._debug = self._te_name+":\n"
         self._effective_cnt = 0
         self._tags = node.get('tag', list())
 
+        path_prefix = self._buildir
+        if self.get_run_attr('path_resolution', True) is False:
+            path_prefix = ""
+            
         for elt_k, elt_v in self._artifacts.items():
             if not os.path.isabs(elt_v):
-                self._artifacts[elt_k] = os.path.join(self._buildir, elt_v)
+                self._artifacts[elt_k] = os.path.join(path_prefix, elt_v)
 
         # allow tags to be given as array OR a single string
         if not isinstance(self._tags, list):
@@ -216,6 +221,20 @@ class TEDescriptor:
         # apply retro-compatibility w/ old syntax
         self._compatibility_support(node.get('_compat', None))
 
+    @staticmethod
+    def get_attr(node, name, dflt=None):
+        if 'attributes' in node and name in node['attributes'].keys():
+            return node['attributes'][name]
+        else:
+            return dflt
+
+    def get_run_attr(self, name, default=None):
+        return TEDescriptor.get_attr(self._run, name, default)
+
+    def get_build_attr(self, name, default=None):
+        return TEDescriptor.get_attr(self._build, name, default)
+    
+
     def _compatibility_support(self, compat):
         """Convert tricky keywords from old syntax too complex to be handled
         by the automatic converter.
@@ -310,7 +329,7 @@ class TEDescriptor:
         self._build.sources.binary = binary
 
         command = "{cc} {var} {cflags} {files} {ldflags} {out}".format(
-            cc=MetaConfig.root.compiler.commands.get(lang, 'echo'),
+            cc=MetaConfig.root.compiler.commands.get(lang, ''),
             var=prepare_cmd_build_variants(self._build.variants),
             cflags=self._build.get('cflags', ''),
             files=" ".join(self._build.files),
@@ -492,6 +511,7 @@ class TEDescriptor:
             delta=self._validation.time.get("tolerance", 0),
             rc=self._validation.get("expect_exit", 0),
             artifacts=self._artifacts,
+            analysis=self._validation.get("analysis", {}),
             resources=1,
             wd=chdir
         )
@@ -530,18 +550,26 @@ class TEDescriptor:
 
             # attempt to determine test working directory
             chdir = self._run.cwd if self._run.cwd else self._buildir
+            
+                
             if not os.path.isabs(chdir):
                 chdir = os.path.abspath(os.path.join(self._buildir, chdir))
 
-            program = os.path.abspath(os.path.join(chdir, program))
+            # keep the original value if user disabled prefix resolution
+            if self.get_run_attr('path_resolution', True) is True:
+                program = os.path.abspath(os.path.join(chdir, program))
 
-            command = "{runtime} {runtime_args} {args} {program} {params}".format(
-                runtime=MetaConfig.root.runtime.program,
-                runtime_args=MetaConfig.root.runtime.get('args', ''),
-                args=" ".join(args),
-                program=program,
-                params=" ".join(params)
-            )
+            command = "{program} {params}".format(
+                    program=program,
+                    params=" ".join(params)
+                )
+            if self.get_run_attr('command_wrap', True) is True:
+                command = "{runtime} {runtime_args} {args} {cmd}".format(
+                    runtime=MetaConfig.root.runtime.get('program', ''),
+                    runtime_args=MetaConfig.root.runtime.get('args', ''),
+                    args=" ".join(args),
+                    cmd=command
+                )
 
             self._effective_cnt += 1
 
@@ -553,12 +581,14 @@ class TEDescriptor:
                 job_deps=te_job_deps,
                 mod_deps=te_mod_deps,
                 tags=self._tags,
+                metrics=self._metrics,
                 environment=env,
                 dim=comb.get('n_node', 1),
                 time=self._validation.time.get("mean", 0),
                 delta=self._validation.time.get("tolerance", 0),
                 rc=self._validation.get("expect_exit", 0),
                 valscript=self._validation.script.get('path', None),
+                analysis=self._validation.get("analysis", {}),
                 comb=comb,
                 wd=chdir,
                 artifacts=self._artifacts,
@@ -578,7 +608,11 @@ class TEDescriptor:
         if self._build:
             yield from self.__construct_compil_tests()
         if self._run:
-            self._serie = Serie({**self._criterion, **self._program_criterion})
+                
+            if self.get_run_attr('command_wrap', True) is False:
+                self._serie = Serie({**self._program_criterion})
+            else:
+                self._serie = Serie({**self._criterion, **self._program_criterion})
             yield from self.__construct_runtime_tests()
             del self._serie
 
