@@ -2,11 +2,14 @@ import base64
 import json
 import os
 import shlex
+import re
 from enum import IntEnum
 
 from pcvs.helpers import log
 from pcvs.helpers.criterion import Combination
-from pcvs.helpers.system import ValidationScheme
+from pcvs.helpers.system import MetaConfig, ValidationScheme
+from pcvs.helpers.utils import Program
+from pcvs.plugins import Plugin
 
 
 class Test:
@@ -25,9 +28,11 @@ class Test:
     :cvar str NOSTART_STR: constant, setting default output when job cannot be run.
     """
     Timeout_RC = 127
+    SCHED_MAX_ATTEMPTS = 50
     res_scheme = ValidationScheme("test-result")
 
     NOSTART_STR = b"This test cannot be started."
+    MAXATTEMPTS_STR = b"This test has failed to be scheduled too many times. Discarded."
 
     class State(IntEnum):
         """Provide Status management, specifically for tests/jobs.
@@ -52,6 +57,7 @@ class Test:
         FAILURE = 3
         ERR_DEP = 4
         ERR_OTHER = 5
+        EXECUTED = 6
 
         def __str__(self):
             """Stringify to return the label.
@@ -99,14 +105,16 @@ class Test:
             suffix=kwargs.get('user_suffix'))
 
         self._execmd = kwargs.get('command', '')
+    
         self._data = {
-            'metrics': None,
+            'metrics': kwargs.get('metrics', {}),
             'tags': kwargs.get('tags', []),
             'artifacts': kwargs.get('artifacts', {}),
-
         }
+
         self._validation = {
             'matchers': kwargs.get('matchers'),
+            'analysis': kwargs.get('analysis'),
             'script': kwargs.get('valscript'),
             'expect_rc': kwargs.get('rc', 0),
             'time': kwargs.get('time', 0),
@@ -116,6 +124,8 @@ class Test:
         self._mod_deps = kwargs.get("mod_deps", [])
         self._depnames = kwargs.get('job_deps', [])
         self._deps = []
+        self._invocation_cmd = self._execmd
+        self._sched_cnt = 0
 
     @property
     def tags(self):
@@ -229,7 +239,13 @@ class Test:
         :rtype: list
         """
         return self._mod_deps
-
+    
+    def get_dep_graph(self):
+        res = {}
+        for d in self._deps:
+            res[d.name] = d.get_dep_graph()
+        return res
+        
     def resolve_a_dep(self, name, obj):
         """Resolve the dep object for a given dep name.
 
@@ -273,7 +289,7 @@ class Test:
         :rtype: :class:`Test` or NoneType
         """
         for d in self._deps:
-            if d.has_completed_deps() and d.state == Test.State.WAITING:
+            if d.state == Test.State.WAITING:
                 return d
         return None
 
@@ -320,17 +336,79 @@ class Test:
         """
         if state is None:
             state = Test.State.SUCCESS if self._validation['expect_rc'] == rc else Test.State.FAILURE
-        self.executed(state)
-        self._rc = rc
-        self._output = base64.b64encode(out).decode('ascii')
-        self._exectime = time
-
+        
+        self.save_raw_run(rc=rc, out=out, time=time)
+        self.save_status(state)
+        
         for elt_k, elt_v in self._data['artifacts'].items():
             if os.path.isfile(elt_v):
                 with open(elt_v, 'rb') as fh:
                     self._data['artifacts'][elt_k] = base64.b64encode(
-                        fh.read()).decode("ascii")
+                        fh.read()).decode("utf-8")
 
+    def save_raw_run(self, out=None, rc=None, time=None):
+        """TODO:
+        """
+        if rc is not None:
+            self._rc = rc
+        if out is not None:
+            self._output = base64.b64encode(out).decode('utf-8')
+        if time is not None:
+            self._exectime = time
+
+    def extract_metrics(self):
+        """TODO:
+        """
+        raw_output = base64.b64decode(self._output).decode("utf-8")
+        for name in self._data['metrics'].keys():
+            node = self._data['metrics'][name]
+            
+            try:
+                ens = set if node['attributes']['unique'] else list
+            except KeyError:
+                ens = list
+            
+            self._data['metrics'][name]['values'] = list(ens(re.findall(node['key'], raw_output)))
+
+    def evaluate(self):
+        """TODO:
+        """
+        state = Test.State.SUCCESS
+        
+        if self._validation['expect_rc'] != self._rc:
+            state = Test.State.FAILURE
+        
+        raw_output = base64.b64decode(self._output).decode('utf-8')
+        
+        # if test should be validated through a matching regex
+        if state == Test.State.SUCCESS and self._validation['matchers'] is not None :
+            for k, v in self._validation['matchers'].items():
+                expected = (v.get('expect', True) is True)
+                found = re.search(v['expr'], raw_output)
+                if (found and not expected) or (not found and expected):
+                    state = Test.State.FAILURE
+                    break
+
+        if state == Test.State.SUCCESS and self._validation['analysis'] is not None:
+            analysis = self._validation['analysis']
+            args = self._validation.get('args', {})
+            s = MetaConfig.root.get_internal("pColl").invoke_plugins(
+                Plugin.Step.TEST_RESULT_EVAL, analysis=analysis, job=self)
+            if s is not None:
+                state = s
+            
+
+        # if a custom script is provided
+        if state == Test.State.SUCCESS and self._validation['script'] is not None:
+            p = Program(self._validation['script'])
+            p.run()
+            if self._validation['expect_rc'] != p.rc:
+                state = Test.State.FAILURE
+        self._state = state
+        
+    def save_status(self, state):
+        self.executed(state)
+        
     def display(self):
         """Print the Test into stdout (through the manager)."""
         colorname = "yellow"
@@ -342,14 +420,15 @@ class Test:
         elif self._state == Test.State.FAILURE:
             colorname = "red"
             icon = "fail"
-        elif self._state == Test.State.ERR_DEP:
+        elif self._state in [Test.State.ERR_DEP, Test.State.ERR_OTHER]:
             colorname = "yellow"
             icon = "fail"
 
         log.manager.print_job(label, self._exectime, self.name,
                               colorname=colorname, icon=icon)
         if self._output:
-            if (log.manager.has_verb_level("info") and self.state == Test.State.FAILURE) or log.manager.has_verb_level("debug"):
+            if (MetaConfig.root.validation.print_level == 'all' or \
+                (self.state == Test.State.FAILURE) and MetaConfig.root.validation.print_level == 'errors'):
                 log.manager.print(base64.b64decode(self._output))
 
     def executed(self, state=None):
@@ -373,6 +452,12 @@ class Test:
         """Flag the job as picked up for scheduling."""
         self._state = Test.State.IN_PROGRESS
 
+    def not_picked(self):
+        self._sched_cnt += 1
+
+    def pick_count(self):
+        return self._sched_cnt
+
     @property
     def state(self):
         """Getter for current job state.
@@ -381,6 +466,12 @@ class Test:
         :rtype: :class:`Test.State`
         """
         return self._state
+    
+    @property
+    def time(self):
+        """TODO:
+        """
+        return self._exectime
 
     def to_json(self, strstate=False):
         """Serialize the whole Test as a JSON object.
@@ -408,21 +499,21 @@ class Test:
         """
 
         if isinstance(test_json, str):
-            test_json = json.load(test_json)
+            test_json = json.loads(test_json)
 
         assert(isinstance(test_json, dict))
         self.res_scheme.validate(test_json)
 
-        self._id = test_json.get("id")
-        self._comb = Combination({}, self._id.get('comb'))
-        self._execmd = test_json.get("exec")
-        self._data = test_json.get("data")
+        self._id = test_json.get("id", -1)
+        self._comb = Combination({}, self._id.get('comb', {}))
+        self._execmd = test_json.get("exec", "")
+        self._data = test_json.get("data", "")
 
-        res = test_json.get("result")
-        self._rc = res.get("rc")
-        self._output = res.get("output")
+        res = test_json.get("result", {})
+        self._rc = res.get("rc", -1)
+        self._output = res.get("output", "")
         self._state = Test.State(res.get("state", Test.State.ERR_OTHER))
-        self._exectime = res.get("time")
+        self._exectime = res.get("time", 0)
 
     def generate_script(self, srcfile):
         """Serialize test logic to its Shell representation.
@@ -441,11 +532,11 @@ class Test:
         cmd_code = ""
         post_code = ""
 
-        self._invocation_cmd = 'sh {} {}'.format(srcfile, self._id['fq_name'])
+        self._invocation_cmd = 'bash {} {}'.format(srcfile, self._id['fq_name'])
 
         # if changing directory is required by the test
         if self._cwd is not None:
-            cd_code += "cd '{}'\n".format(shlex.quote(self._cwd))
+            cd_code += "cd '{}'".format(shlex.quote(self._cwd))
 
         # manage package-manager deps
         for elt in self._mod_deps:
@@ -453,55 +544,25 @@ class Test:
 
         # manage environment variables defined in TE
         if self._testenv is not None:
-            for e in self._testenv:
-                env_code += "{}; export {}\n".format(
-                    shlex.quote(e), shlex.quote(e.split('=')[0]))
-
-        # if test should be validated through a matching regex
-        if self._validation['matchers'] is not None:
-            for k, v in self._validation['matchers'].items():
-                expr = v['expr']
-                required = (v.get('expect', True) is True)
-                # if match is required set 'ret' to 1 when grep fails
-                post_code += "{echo} | {grep} '{expr}' {fail} ret=1\n".format(
-                    echo='echo "$output"',
-                    grep='grep -qP',
-                    expr=shlex.quote(expr),
-                    fail='||' if required else "&&")
-
-        # if a custom script is provided (in addition or not to matchers)
-        if self._validation['script'] is not None:
-            post_code += "{echo} | {script}; ret=$?".format(
-                echo='echo "$output"',
-                script=shlex.quote(self._validation['script'])
-            )
+            env_code = "\n".join([
+                "{}; export {}".format(shlex.quote(e),
+                                        shlex.quote(e.split('=')[0]))
+                for e in self._testenv
+                ])
 
         cmd_code = self._execmd
 
         return """
         "{name}")
-            if test -n "$PCVS_SHOW"; then
-                test "$PCVS_SHOW" = "env" -o "$PCVS_SHOW" = "all" &&  echo '{p_env}'
-                test "$PCVS_SHOW" = "loads" -o "$PCVS_SHOW" = "all" &&  echo '{p_pm}'
-                test "$PCVS_SHOW" = "cmd" -o "$PCVS_SHOW" = "all" &&  echo {p_cmd}
-                exit 0
-            fi
             {cd_code}
-            {pm_code}
-            {env_code}
-            output=$({cmd_code} 2>&1)
-            ret=$?
-            test -n "$output" && echo "$output"
-            {post_code}
+            pcvs_load={pm_code}
+            pcvs_env={env_code}
+            pcvs_cmd={cmd_code}
             ;;""".format(
-            p_cmd="{}".format(shlex.quote(cmd_code)),
-            p_env="{}".format(env_code),
-            p_pm="{}".format(pm_code),
+            cmd_code="{}".format(shlex.quote(cmd_code)),
+            env_code="{}".format(shlex.quote(env_code)),
+            pm_code="{}".format(shlex.quote(pm_code)),
             cd_code=cd_code,
-            pm_code=pm_code,
-            env_code=env_code,
-            cmd_code=cmd_code,
-            post_code=post_code,
             name=self._id['fq_name']
         )
 
